<?php
	/*
	* 2018 - VEGA Informatique
	*
	* NOTICE OF LICENSE
	*
	* This source file is subject to the Open Software License (OSL 3.0)
	* that is bundled with this package in the file LICENSE.txt.
	* It is also available through the world-wide-web at this URL:
	* http://opensource.org/licenses/osl-3.0.php
	*
	* DISCLAIMER
	*
	* Do not edit or add to this file if you wish to upgrade PolarisLink to newer
	* versions in the future. 
	*
	*  @author VEGA Informatique <contact@vega-info.fr>
	*  @copyright  2018 VEGA Informatique
	*  @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
	*/

	if (!defined('_PS_VERSION_'))
		exit;
			
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkAPI.php");
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/PolarisLinkJobManager.php");
	require_once(_PS_ROOT_DIR_ . "/modules/polarislink/classes/VegaAutoUpdate.php");
	
	/*
	 * soumission des travaux Polaris
	 *
	 */
	class PolarisLinkAPI_enqueueJobModuleFrontController extends PolarisLinkAPI
	{
		protected function runAPI()
		{
			// Surtout plus de cache !!!
			Db::getInstance()->disableCache();

			$id_job = isset($_REQUEST['id']) ? trim($_REQUEST['id']) : null;
			$partial = isset($_REQUEST['partial']) ? $_REQUEST['partial'] : false;
			
			$jobFilename = _PS_ROOT_DIR_."/polarislink/jobs/tmp/lk-".$this->Link->LinkID;
			$outStream = null;
	
			if (empty($id_job))
			{
				// On s'assure que le répertoire temporaire existe
				if (!file_exists($jobFilename))
					mkdir($jobFilename, 0700, true);
				
				// Ouverture du fichier
				$id_job = uniqid(sha1(gethostname()));
				$jobFilename .= "/job-".$id_job;
				
				if (!$outStream = fopen($jobFilename, "w"))
					throw new Exception("impossible d'ouvrir un fichier de stockage pour le travail");
			}
			else
			{
				// On sécurise l'id du fichier transmis (on ne se balade pas sur le serveur)
				if (!preg_match("/^([a-zA-Z0-9]+)$/", $id_job))
					throw new Exception("nom de travail incorrect !");
			
				// Reprise, on localise le job !
				$jobFilename .= "/job-".$id_job;
				
				if (!file_exists($jobFilename))
					throw new Exception(sprintf("le travail %s est introuvable !", $id_job));
				
				// Reprise des données, le champs startAt doit être positionné !!
				$startAt = isset($_REQUEST['startAt']) ? $_REQUEST['startAt'] : 0;
				if (empty($startAt))
					throw new Exception("en cas de reprise de téléchargement, vous devez préciser le point de reprise avec «startAt»");
				
				if ($startAt > filesize($jobFilename))
					throw new Exception("les données de reprise ont l'air erronées");
				
				if (!$outStream = fopen($jobFilename, "c"))
					throw new Exception("impossible d'ouvrir un fichier de stockage pour le travail");
				
				fseek($outStream, $startAt, SEEK_SET);
			}
						
			$inStream = fopen("php://input", "r");
			stream_copy_to_stream($inStream, $outStream);			
			fclose($outStream);

			if (!$partial)
			{
				clearstatcache();
				$size = filesize($jobFilename);
				
				// On bouge le travail à faire dedans ?				
				$jobsManager = new PolarisLinkJobManager($this->Link);				
				$jobsManager->enqueueJob($jobFilename);				

				if ($this->AnswerAndContinueOffline(array("id" => $id_job, "len" => $size)))
				{
					// Uniquement si nous avons pu passer en mode thread, sinon le script va retenir la réponse
					// à l'envoi du fichier, ce qui peut-être problématique
					// Ce n'est pas grave, le traitement sera différé jusqu'au prochain appel de api_run
					ob_start();

					// Scan jobs
					$jobsManager->scanJobs();

					// On traite en arrière plan après avoir répondu
					if (!Configuration::get("POLARISLINK_CRON_DATAUPDATE"))
					{
						$jobsManager->run();					
					}
				
					ob_end_clean();

					// Appel métriques ?
					$this->module->sendMetrics();
				}
				else
				{					
					// Autre solution, on est pas en fpm, on triche : on va appeler le api_run en curl sur un autre processus, ce qui va déclencher
					// un traitement en fond, non attaché à qqchose
					try
					{
						$ch = curl_init();

						$url = Tools::getHttpHost(true) . str_replace("api_enqueueJob", "api_run", $_SERVER["REQUEST_URI"]);

						// Configuration de l'URL et d'autres options
						$defaults = array(
							CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
							CURLOPT_URL => $url,
							CURLOPT_HEADER => 0,   
							CURLOPT_HTTPHEADER => [ 'Authorization: Basic '.base64_encode($this->Link->AccessKey) ], 
							CURLOPT_RETURNTRANSFER => TRUE,
							CURLOPT_TIMEOUT => 15
						);
						
						curl_setopt_array($ch, $defaults);

						// Récupération de l'URL et affichage sur le navigateur
						curl_exec($ch);						

						// Fermeture de la session cURL
						curl_close($ch);
					}
					catch (Exception $ex)
					{
						// pas grave
					}
				}
			}
			else
			{			
				clearstatcache();
				$this->answerJSON(array(
								"id" => $id_job, 
								"len" => filesize($jobFilename)));
			}
			return true;
		}
	}
 